@@ -26,7 +26,7 @@
 #include <string.h>
 #include <libcob.h>
 
-#if !(defined __CYGWIN__ || defined __MINGW32__)
+#ifndef __MINGW32__
 #include <unistd.h>
 #endif
 
@@ -38,7 +38,7 @@
 #include "tree.h"
 #include "lib/getopt.h"
 
-
+
 /*
  * Global variables
  */
@@ -84,7 +84,7 @@ struct cb_program *current_program = NULL;
 struct cb_statement *current_statement = NULL;
 struct cb_label *current_section = NULL, *current_paragraph = NULL;
 
-
+
 /*
  * Local variables
  */
@@ -92,6 +92,8 @@ struct cb_label *current_section = NULL, *current_paragraph = NULL;
 static int save_temps = 0;
 static int verbose_output = 0;
 
+static int	strip_output = 0;
+
 static char *program_name;
 static char *output_name;
 
@@ -109,11 +111,19 @@ static struct filename {
   char source[FILENAME_MAX];			/* foo.cob */
   char preprocess[FILENAME_MAX];		/* foo.i */
   char translate[FILENAME_MAX];			/* foo.c */
+  char trstorage[FILENAME_MAX];			/* foo.c.h */
   char object[FILENAME_MAX];			/* foo.o */
   struct filename *next;
 } *file_list;
 
-
+
+#if defined (__GNUC__) && (__GNUC__ >= 3)
+static const char	fcopts[] =
+	" -finline-functions -fno-gcse -freorder-blocks";
+#else
+static const char	fcopts[] = " ";
+#endif
+
 struct cb_text_list *
 cb_text_list_add (struct cb_text_list *list, const char *text)
 {
@@ -131,7 +141,7 @@ cb_text_list_add (struct cb_text_list *list, const char *text)
     }
 }
 
-
+
 /*
  * Local functions
  */
@@ -162,6 +172,9 @@ init_environment (int argc, char *argv[])
 
   init_var (tmpdir,      "TMPDIR",     "/tmp");
   init_var (cob_cc,      "COB_CC",     COB_CC);
+#if defined (__GNUC__) && (__GNUC__ >= 3)
+  strcat (cob_cc, " -pipe");
+#endif
   init_var (cob_cflags,  "COB_CFLAGS", COB_CFLAGS);
   init_var (cob_libs,    "COB_LIBS",   COB_LIBS);
   init_var (cob_ldflags, "COB_LDFLAGS", "");
@@ -183,7 +196,7 @@ terminate (const char *str)
   exit (1);
 }
 
-
+
 /*
  * Command line
  */
@@ -205,6 +218,7 @@ static struct option long_options[] = {
   {"static", no_argument, &cb_flag_static_call, 1},
   {"dynamic", no_argument, &cb_flag_static_call, 0},
   {"O2", no_argument, 0, '2'},
+  {"Os", no_argument, 0, 's'},
   {"MT", required_argument, 0, '%'},
   {"MF", required_argument, 0, '@'},
 #undef CB_FLAG
@@ -252,7 +266,7 @@ print_usage (void)
 "                          v023        Open Cobol V23 Compatible (deprecated)\n"
 "                          default     When not specified\n"
 "                        See config/default.conf and config/*.conf\n"
-"  -O, -O2               Enable optimization\n"
+"  -O, -O2, -Os          Enable optimization\n"
 "  -g                    Produce debugging information in the output\n"
 "  -debug                Enable all run-time error checking\n"
 "  -o <file>             Place the output into <file>\n"
@@ -316,11 +330,21 @@ process_command_line (int argc, char *argv[])
 	case 'O':
 	  cb_flag_runtime_inlining = 1;
 	  strcat (cob_cflags, " -O");
+	  strcat (cob_cflags, fcopts);
 	  break;
 
 	case '2': /* -O2 */
 	  cb_flag_runtime_inlining = 1;
+	  strip_output = 1;
 	  strcat (cob_cflags, " -O2");
+	  strcat (cob_cflags, fcopts);
+	  break;
+
+	case 's': /* -Os */
+	  cb_flag_runtime_inlining = 1;
+	  strip_output = 1;
+	  strcat (cob_cflags, " -Os");
+	  strcat (cob_cflags, fcopts);
 	  break;
 
 	case 'g':
@@ -415,6 +439,8 @@ process_command_line (int argc, char *argv[])
 	exit (1);
       }
 
+  strcat (cob_cflags, " -fsigned-char");
+
   if (cb_compile_level == CB_LEVEL_EXECUTABLE)
     cb_flag_main = 1;
 
@@ -575,8 +601,13 @@ preprocess (struct filename *fn)
 	terminate (fn->preprocess);
     }
 
-  if (ppopen (fn->source, NULL) != 0)
+  if (ppopen (fn->source, NULL) != 0) {
+    if ( ppout != stdout ) {
+	fclose(ppout);
+	unlink(fn->preprocess);
+    }
     exit (1);
+  }
 
   if (verbose_output)
     fprintf (stderr, "preprocessing %s into %s\n",
@@ -647,6 +678,7 @@ process_translate (struct filename *fn)
   /* open the storage file */
   cb_storage_file_name = malloc (strlen (fn->translate) + 3);
   sprintf (cb_storage_file_name, "%s.h", fn->translate);
+  strcpy(fn->trstorage, cb_storage_file_name);
   cb_storage_file = fopen (cb_storage_file_name, "w");
   if (!cb_storage_file)
     terminate (cb_storage_file_name);
@@ -681,16 +713,26 @@ static int
 process_assemble (struct filename *fn)
 {
   char buff[FILENAME_MAX];
-  sprintf (buff, "%s -c -o %s %s %s",
-	   cob_cc, fn->object, cob_cflags, fn->translate);
+
+  if (cb_compile_level == CB_LEVEL_MODULE) {
+	sprintf (buff, "%s -c %s %s -o %s %s",
+		cob_cc, cob_cflags, COB_PIC_FLAGS, fn->object, fn->translate);
+  } else {
+	sprintf (buff, "%s -c %s -o %s %s",
+		cob_cc, cob_cflags, fn->object, fn->translate);
+  }
   return process (buff);
 }
 
 static int
 process_module (struct filename *fn)
 {
+#ifdef	COB_STRIP_CMD
+  int	ret;
+#endif
   char buff[FILENAME_MAX];
   char name[FILENAME_MAX];
+
   if (output_name)
     strcpy (name, output_name);
   else
@@ -700,18 +742,33 @@ process_module (struct filename *fn)
       strcat (name, COB_MODULE_EXT);
     }
 #if (defined __CYGWIN__ || defined __MINGW32__)
-  sprintf (buff, "%s -shared -Wl,--enable-auto-import %s -o %s %s %s",
-	   cob_cc, cob_ldflags, name, fn->object, cob_libs);
+  sprintf (buff, "%s %s %s %s %s -o %s %s %s",
+	cob_cc, COB_SHARED_OPT, cob_ldflags, COB_PIC_FLAGS,
+	COB_EXPORT_DYN, name, fn->object, cob_libs);
 #else
-  sprintf (buff, "%s -shared %s -o %s %s",
-	   cob_cc, cob_ldflags, name, fn->object);
+  sprintf (buff, "%s %s %s %s %s -o %s %s",
+	cob_cc, COB_SHARED_OPT, cob_ldflags, COB_PIC_FLAGS,
+	COB_EXPORT_DYN, name, fn->object);
 #endif
+
+#ifdef	COB_STRIP_CMD
+  ret  = process (buff);
+  if ( strip_output && ret == 0 ) {
+	sprintf(buff, "%s %s", COB_STRIP_CMD, name);
+  	ret  = process (buff);
+  }
+  return ret;
+#else
   return process (buff);
+#endif
 }
 
 static int
 process_link (struct filename *l)
 {
+#ifdef	COB_STRIP_CMD
+  int	ret;
+#endif
   char buff[FILENAME_MAX], objs[FILENAME_MAX] = "";
   char name[FILENAME_MAX];
 
@@ -725,14 +782,19 @@ process_link (struct filename *l)
   if (output_name)
     strcpy (name, output_name);
 
-#if (defined __CYGWIN__ || defined __MINGW32__)
-  sprintf (buff, "%s -Wl,--export-all-symbols -Wl,--enable-auto-import %s -o %s %s %s",
-	   cob_cc, cob_ldflags, name, objs, cob_libs);
+  sprintf (buff, "%s %s %s -o %s %s %s",
+	   cob_cc, cob_ldflags, COB_EXPORT_DYN, name, objs, cob_libs);
+
+#ifdef	COB_STRIP_CMD
+  ret  = process (buff);
+  if ( strip_output && ret == 0 ) {
+	sprintf(buff, "%s %s", COB_STRIP_CMD, name);
+  	ret  = process (buff);
+  }
+  return ret;
 #else
-  sprintf (buff, "%s -rdynamic %s -o %s %s %s",
-	   cob_cc, cob_ldflags, name, objs, cob_libs);
-#endif
   return process (buff);
+#endif
 }
 
 int
@@ -767,6 +829,7 @@ main (int argc, char *argv[])
       fn->next = file_list;
       file_list = fn;
 
+      cb_id = 1;
       /* Preprocess */
       if (cb_compile_level >= CB_LEVEL_PREPROCESS && fn->need_preprocess)
 	if (preprocess (fn) != 0)
@@ -817,7 +880,7 @@ main (int argc, char *argv[])
 	      && (status == 1 || cb_compile_level > CB_LEVEL_TRANSLATE))
 	    {
 	      remove (fn->translate);
-	      remove (cb_storage_file_name);
+	      remove (fn->trstorage);
 	    }
 	  if (fn->need_assemble
 	      && (status == 1 || cb_compile_level > CB_LEVEL_ASSEMBLE))
