@@ -30,6 +30,12 @@
 #include "tree.h"
 
 static int param_id = 0;
+static int loop_counter = 0;
+static int progid = 0;
+static int last_line = 0;
+static int needs_exit_prog = 0;
+int i_counters[8] = {0, 0, 0, 0, 0, 0, 0, 0};
+int entry_number = 0;
 
 static void output (const char *fmt, ...)
      __attribute__ ((__format__ (__printf__, 1, 2)));
@@ -43,7 +49,7 @@ static void output_integer (cb_tree x);
 static void output_index (cb_tree x);
 static void output_func_1 (const char *name, cb_tree x);
 
-
+
 /*
  * Output routines
  */
@@ -51,6 +57,30 @@ static void output_func_1 (const char *name, cb_tree x);
 static int output_indent_level = 0;
 static FILE *output_target;
 
+static struct attr_list {
+	struct attr_list	*next;
+	int			id;
+	char			type;
+	char			digits;
+	char			scale;
+	char			flags;
+	unsigned char		*pic;
+} *attr_cache = NULL;
+
+static struct literal_list {
+	struct literal_list	*next;
+	int			id;
+	struct cb_literal	*literal;
+	cb_tree			x;
+} *literal_cache = NULL;
+
+static struct field_list {
+	struct field_list	*next;
+	struct cb_field		*f;
+	cb_tree			x;
+} *field_cache = NULL;
+
+
 static void
 output (const char *fmt, ...)
 {
@@ -66,8 +96,9 @@ output (const char *fmt, ...)
 static void
 output_newline (void)
 {
-  if (output_target)
+  if (output_target) {
     fputs ("\n", output_target);
+  }
 }
 
 static void
@@ -104,12 +135,12 @@ output_indent (const char *str)
   for (p = str; *p == ' '; p++)
     level++;
 
-  if (*p == '}')
+  if (*p == '}' && strcmp(str, "})") != 0)
     output_indent_level -= level;
 
   output_line (str);
 
-  if (*p == '{')
+  if (*p == '{' && strcmp(str, ")}") != 0)
     output_indent_level += level;
 }
 
@@ -148,7 +179,7 @@ output_storage (const char *fmt, ...)
   va_end (ap);
 }
 
-
+
 /*
  * Field
  */
@@ -178,12 +209,14 @@ output_base (struct cb_field *f)
 
   if (!f01->flag_base)
     {
-      if (!f01->flag_external && !f01->flag_local)
-	output_storage ("static ");
-      output_storage ("unsigned char %s%s[%d]  __attribute__ ((__aligned__(8)));",
-		      CB_PREFIX_BASE, name, f01->memory_size);
-      output_storage ("\t/* %s */\n", f01->name);
-      f01->flag_base = 1;
+	if ( !f01->flag_external ) {
+		if (!f01->flag_local)
+			output_storage ("static ");
+		output_storage ("unsigned char %s%s[%d]\t__attribute__ ((__aligned__(8)));",
+				CB_PREFIX_BASE, name, f01->memory_size);
+		output_storage ("  /* %s */\n", f01->name);
+	}
+	f01->flag_base = 1;
     }
   output ("%s%s", CB_PREFIX_BASE, name);
 
@@ -323,16 +356,6 @@ output_size (cb_tree x)
 static int
 lookup_attr (char type, char digits, char scale, char flags, unsigned char *pic)
 {
-  static struct attr_list {
-    int id;
-    char type;
-    char digits;
-    char scale;
-    char flags;
-    unsigned char *pic;
-    struct attr_list *next;
-  } *attr_cache = NULL;
-
   struct attr_list *l;
 
   /* search attribute cache */
@@ -346,19 +369,6 @@ lookup_attr (char type, char digits, char scale, char flags, unsigned char *pic)
       return l->id;
 
   /* output new attribute */
-  output_storage ("static cob_field_attr %s%d = ", CB_PREFIX_ATTR, cb_id);
-  output_storage ("{%d, %d, %d, %d, ", type, digits, scale, flags);
-  if (pic)
-    {
-      unsigned char *s;
-      output_storage ("\"");
-      for (s = pic; *s; s += 2)
-	output_storage ("%c\\%03o", s[0], s[1]);
-      output_storage ("\"");
-    }
-  else
-    output_storage ("0");
-  output_storage ("};\n");
 
   /* cache it */
   l = malloc (sizeof (struct attr_list));
@@ -463,7 +473,7 @@ output_field (cb_tree x)
   output ("}");
 }
 
-
+
 /*
  * Literal
  */
@@ -471,11 +481,6 @@ output_field (cb_tree x)
 static int
 lookup_literal (cb_tree x)
 {
-  static struct literal_list {
-    int id;
-    struct cb_literal *literal;
-    struct literal_list *next;
-  } *literal_cache = NULL;
   
   struct cb_literal *literal = CB_LITERAL (x);
   struct literal_list *l;
@@ -494,23 +499,20 @@ lookup_literal (cb_tree x)
   output_target = 0;
   output_field (x);
 
-  output_target = cb_storage_file;
-  output ("static cob_field %s%d = ", CB_PREFIX_CONST, cb_id);
-  output_field (x);
-  output (";\n");
   output_target = yyout;
 
   /* cache it */
   l = malloc (sizeof (struct literal_list));
   l->id = cb_id;
   l->literal = literal;
+  l->x = x;
   l->next = literal_cache;
   literal_cache = l;
 
   return cb_id++;
 }
 
-
+
 /*
  * Integer
  */
@@ -523,6 +525,8 @@ output_integer (cb_tree x)
     case CB_TAG_CONST:
       if (x == cb_zero)
 	output ("0");
+      else if ( x == cb_null )
+	output ("(unsigned char *)0");
       else
 	output ("%s", CB_CONST (x)->val);
       break;
@@ -574,6 +578,7 @@ output_integer (cb_tree x)
 	switch (f->usage)
 	  {
 	  case CB_USAGE_INDEX:
+	  case CB_USAGE_LENGTH:
 	    output ("(*(int *) (");
 	    output_data (x);
 	    output ("))");
@@ -626,7 +631,7 @@ output_integer (cb_tree x)
 		  {
 		  case 1: output ("char"); break;
 		  case 2: output ("short"); break;
-		  case 4: output ("long"); break;
+		  case 4: output ("int"); break;
 		  case 8: output ("long long"); break;
 		  }
 		output (" *) (");
@@ -667,7 +672,7 @@ output_index (cb_tree x)
     }
 }
 
-
+
 /*
  * Parameter
  */
@@ -753,23 +758,27 @@ output_param (cb_tree x, int id)
 	if (r->check)
 	  {
 	    cb_tree l;
-	    output_indent ("({");
+	    output_indent (" ({");
 	    for (l = r->check; l; l = CB_CHAIN (l))
 	      output_stmt (CB_VALUE (l));
 	  }
 
 	if (!r->subs && !r->offset && !f->flag_local && f->count > 0
+	    && !f->flag_external
 	    && !cb_field_variable_size (f) && !cb_field_variable_address (f))
 	  {
 	    if (!f->flag_field)
 	      {
+		struct field_list *l;
+
 		output_target = 0;
 		output_field (x);
 
-		output_target = cb_storage_file;
-		output ("static cob_field %s%d = ", CB_PREFIX_FIELD, f->id);
-		output_field (x);
-		output ("; /* %s */\n", f->name);
+		l = malloc (sizeof (struct field_list));
+		l->x = x;
+		l->f = f;
+		l->next = field_cache;
+		field_cache = l;
 
 		f->flag_field = 1;
 		output_target = yyout;
@@ -788,11 +797,13 @@ output_param (cb_tree x, int id)
 	break;
       }
     default:
+      fprintf(stderr, "Unexpected tree tag %d\n", CB_TREE_TAG (x));
+      fflush(stderr);
       ABORT ();
     }
 }
 
-
+
 /*
  * Function call
  */
@@ -820,7 +831,7 @@ output_func_1 (const char *name, cb_tree x)
   output (")");
 }
 
-
+
 /*
  * Condition
  */
@@ -903,7 +914,7 @@ output_cond (cb_tree x, int save_flag)
     }
 }
 
-
+
 /*
  * MOVE
  */
@@ -914,7 +925,7 @@ output_move (cb_tree src, cb_tree dst)
   output_stmt (cb_build_move (src, dst));
 }
 
-
+
 /*
  * INITIALIZE
  */
@@ -923,10 +934,14 @@ output_move (cb_tree src, cb_tree dst)
 #define INITIALIZE_ONE		1
 #define INITIALIZE_DEFAULT	2
 #define INITIALIZE_COMPOUND	3
+#define INITIALIZE_EXTERNAL	4
 
 static int
 initialize_type (struct cb_initialize *p, struct cb_field *f)
 {
+  if (f->flag_external)
+    return INITIALIZE_EXTERNAL;
+
   if (f->redefines)
     return INITIALIZE_NONE;
 
@@ -996,13 +1011,40 @@ initialize_uniform_char (struct cb_field *f)
     }
 }
 
+static void
+output_initialize_external (cb_tree x, struct cb_field *f)
+{
+  char	*p;
+  char	name[CB_MAX_CNAME];
+
+  output_prefix ();
+  output_data (x);
+  if ( f->ename ) {
+    output (" = cob_external_addr (\"%s\", %d);\n", f->ename, f->size);
+  } else {
+    strcpy(name, f->name);
+    for ( p = name; *p; p++ ) {
+	if ( islower(*p) ) {
+		*p = toupper(*p);
+	}
+    }
+    output (" = cob_external_addr (\"%s\", %d);\n", name, f->size);
+  }
+}
+
 static void
 output_initialize_uniform (cb_tree x, int c, int size)
 {
   output_prefix ();
-  output ("memset (");
-  output_data (x);
-  output (", %d, %d);\n", c, size);
+  if ( size == 1 ) {
+	output ("*(unsigned char *)(");
+	output_data (x);
+	output (") = %d;\n", c);
+  } else {
+	output ("memset (");
+	output_data (x);
+	output (", %d, %d);\n", c, size);
+  }
 }
 
 static void
@@ -1024,9 +1066,15 @@ output_initialize_one (struct cb_initialize *p, cb_tree x)
 	     has VALUE SPACE because cob_build_move doubly checks the value.
 	     We should instead check the value only once.  */
 	  output_prefix ();
-	  output ("memset (");
-	  output_data (x);
-	  output (", ' ', %d);\n", f->size);
+	  if ( f->size == 1 ) {
+		output ("*(unsigned char *)(");
+		output_data (x);
+		output (") = ' ';\n");
+	  } else {
+		output ("memset (");
+		output_data (x);
+		output (", ' ', %d);\n", f->size);
+	  }
 	}
       else if (CB_CONST_P (value)
 	  || CB_TREE_CLASS (value) == CB_CLASS_NUMERIC
@@ -1052,11 +1100,20 @@ output_initialize_one (struct cb_initialize *p, cb_tree x)
 	      memset (buff + l->size, ' ', f->size - l->size);
 	    }
 	  output_prefix ();
-	  output ("memcpy (");
-	  output_data (x);
-	  output (", ");
-	  output_string (buff, f->size);
-	  output (", %d);\n", f->size);
+	  if (f->size == 1)
+	    {
+		output ("*(unsigned char *) (");
+		output_data (x);
+		output (") = %d;\n", *(unsigned char *) buff);
+	    }
+	  else
+	    {
+		output ("memcpy (");
+		output_data (x);
+		output (", ");
+		output_string (buff, f->size);
+		output (", %d);\n", f->size);
+	    }
 	}
       return;
     }
@@ -1071,7 +1128,7 @@ output_initialize_one (struct cb_initialize *p, cb_tree x)
 	}
 
   /* initialize by defualt */
-  if (p->def)
+  if (p->def) {
     switch (CB_TREE_CATEGORY (x))
       {
       case CB_CATEGORY_NUMERIC:
@@ -1085,6 +1142,7 @@ output_initialize_one (struct cb_initialize *p, cb_tree x)
       default:
 	ABORT ();
       }
+  }
 }
 
 static void
@@ -1166,6 +1224,9 @@ output_initialize (struct cb_initialize *p)
     case INITIALIZE_ONE:
       output_initialize_one (p, p->var);
       break;
+    case INITIALIZE_EXTERNAL:
+	  output_initialize_external (p->var, f);
+      break;
     case INITIALIZE_DEFAULT:
       {
 	int c = initialize_uniform_char (f);
@@ -1181,7 +1242,7 @@ output_initialize (struct cb_initialize *p)
     }
 }
 
-
+
 /*
  * SEARCH
  */
@@ -1310,7 +1371,7 @@ output_search (struct cb_search *p)
     output_search_whens (p->table, p->var, p->end_stmt, p->whens);
 }
 
-
+
 /*
  * CALL
  */
@@ -1354,18 +1415,12 @@ output_call (struct cb_call *p)
 	{
 	case CB_CALL_BY_CONTENT:
 	  output_prefix ();
-	  if (CB_NUMERIC_LITERAL_P (x) || x == cb_null)
+	  if (CB_NUMERIC_LITERAL_P (x) || x == cb_null
+		|| ( CB_TREE_CATEGORY (x) == CB_CATEGORY_NUMERIC
+			&& cb_field (x)->usage == CB_USAGE_LENGTH ))
 	    {
 	      output ("*(int *)content_%d = ", n);
-#ifndef WORDS_BIGENDIAN
-	      if (cb_binary_byteorder == CB_BYTEORDER_BIG_ENDIAN)
-		output ("COB_BSWAP_32 (");
-#endif
 	      output_integer (x);
-#ifndef WORDS_BIGENDIAN
-	      if (cb_binary_byteorder == CB_BYTEORDER_BIG_ENDIAN)
-		output (")");
-#endif
 	      output (";\n");
 	    }
 	  else
@@ -1381,6 +1436,13 @@ output_call (struct cb_call *p)
 
   /* function name */
   output_prefix ();
+  n = 0;
+  for (l = p->args; l; l = CB_CHAIN (l))
+    {
+	n++;
+    }
+  output ("cob_call_params = %d;\n", n);
+  output_prefix ();
   if (!dynamic_link)
     {
       /* static link */
@@ -1396,7 +1458,11 @@ output_call (struct cb_call *p)
       else
 	output_funcall (cb_build_funcall_1 ("cob_call_resolve", p->name));
       output (";\n");
+#if defined (__GNUC__) && (__GNUC__ >= 3)
+      output_line ("if ( __builtin_expect((func == NULL), 0) )");
+#else
       output_line ("if (func == NULL)");
+#endif
       output_indent_level += 2;
       output_stmt (p->stmt1);
       output_indent_level -= 2;
@@ -1441,6 +1507,9 @@ output_call (struct cb_call *p)
 		case CB_USAGE_INDEX:
 		  output_integer (x);
 		  break;
+		case CB_USAGE_LENGTH:
+			output_integer (x);
+			break;
 		default:
 		  output ("*");
 		  output_data (x);
@@ -1461,7 +1530,7 @@ output_call (struct cb_call *p)
   output_indent ("}");
 }
 
-
+
 /*
  * GO TO
  */
@@ -1494,12 +1563,15 @@ output_goto (struct cb_goto *p)
       output_indent ("  }");
     }
   else if (p->target == NULL)
-    output_line ("goto exit_program;");
+    {
+       needs_exit_prog = 1;
+       output_line ("goto exit_program;");
+    }
   else
     output_goto_1 (p->target);
 }
 
-
+
 /*
  * PERFORM
  */
@@ -1508,10 +1580,14 @@ static void
 output_perform_call (struct cb_label *lb, struct cb_label *le)
 {
   output_line ("/* PERFORM %s THRU %s */", lb->name, le->name);
-  output_line ("frame_stack[++frame_index] = (struct frame) {%d, &&%s%d};",
-	       le->id, CB_PREFIX_LABEL, cb_id);
+  output_line ("frame_index++;");
+  output_line ("frame_stack[frame_index].perform_through = %d;",
+	       le->id);
+  output_line ("frame_stack[frame_index].return_address = &&%s%d;",
+	       CB_PREFIX_LABEL, cb_id);
   output_line ("goto %s%d;", CB_PREFIX_LABEL, lb->id);
-  output_line ("%s%d:", CB_PREFIX_LABEL, cb_id++);
+  output_line ("%s%d:", CB_PREFIX_LABEL, cb_id);
+  cb_id++;
   output_line ("frame_index--;");
 }
 
@@ -1578,7 +1654,6 @@ output_perform_until (struct cb_perform *p, cb_tree l)
 static void
 output_perform (struct cb_perform *p)
 {
-  static int loop_counter = 0;
 
   switch (p->type)
     {
@@ -1613,7 +1688,7 @@ output_perform (struct cb_perform *p)
     output_stmt (cb_ref (p->exit_label));
 }
 
-
+
 /*
  * Output statement
  */
@@ -1631,7 +1706,6 @@ output_stmt (cb_tree x)
     {
     case CB_TAG_STATEMENT:
       {
-	static int last_line = 0;
 	struct cb_statement *p = CB_STATEMENT (x);
 
 	/* output source location as a comment */
@@ -1654,8 +1728,16 @@ output_stmt (cb_tree x)
 	  }
 
 	if (p->handler1 || p->handler2
-	    || (p->file && CB_EXCEPTION_ENABLE (COB_EC_I_O)))
+	    || (p->file && CB_EXCEPTION_ENABLE (COB_EC_I_O))) {
+
 	  output_line ("cob_exception_code = 0;");
+/* RXW - Dirty must be some other way */
+		if ( p->handler_id == COB_EC_I_O_EOP ) {
+			output_line ("cob_check_eop = 1;");
+		} else {
+			output_line ("cob_check_eop = 0;");
+		}
+	}
 
 	if (p->body)
 	  output_stmt (p->body);
@@ -1667,17 +1749,29 @@ output_stmt (cb_tree x)
 	    if (p->handler1)
 	      {
 		if ((code & 0x00ff) == 0)
+#if defined (__GNUC__) && (__GNUC__ >= 3)
+		  output_line ("if ( __builtin_expect(((cob_exception_code & 0xff00) == 0x%04x), 0) )",
+#else
 		  output_line ("if ((cob_exception_code & 0xff00) == 0x%04x)",
+#endif
 			       code);
 		else
+#if defined (__GNUC__) && (__GNUC__ >= 3)
+		  output_line ("if ( __builtin_expect((cob_exception_code == 0x%04x), 0) )", code);
+#else
 		  output_line ("if (cob_exception_code == 0x%04x)", code);
+#endif
 		output_stmt (p->handler1);
 		if (p->handler2 || p->file)
 		  output_line ("else");
 	      }
 	    if (p->file)
 	      {
+#if defined (__GNUC__) && (__GNUC__ >= 3)
+		output_line ("if ( __builtin_expect(cob_exception_code, 0) )");
+#else
 		output_line ("if (cob_exception_code)");
+#endif
 		output_indent ("  {");
 		output_perform_call (CB_FILE (p->file)->handler,
 				     CB_FILE (p->file)->handler);
@@ -1701,8 +1795,10 @@ output_stmt (cb_tree x)
 	output_line ("/* %s: */", p->name);
 	if (p->need_begin)
 	  output_line ("%s%d:;", CB_PREFIX_LABEL, p->id);
-	if (cb_flag_trace)
+	if (cb_flag_trace) {
 	  output_line ("puts (\"%s\");", p->name);
+	  output_line ("fflush (stdout);");
+	}
 	break;
       }
     case CB_TAG_FUNCALL:
@@ -1784,7 +1880,7 @@ output_stmt (cb_tree x)
     }
 }
 
-
+
 /*
  * File definition
  */
@@ -1799,7 +1895,7 @@ output_file_definition (struct cb_file *f)
       || f->organization == COB_ORG_INDEXED)
     {
       struct cb_alt_key *l;
-      output ("static cob_file_key %s%s[] = {\n", CB_PREFIX_KEYS, f->cname);
+      output ("  static cob_file_key %s%s[] = {\n", CB_PREFIX_KEYS, f->cname);
       output ("  {");
       output_param (f->key, -1);
       output (", 0},\n");
@@ -1814,7 +1910,7 @@ output_file_definition (struct cb_file *f)
     }
 
   /* output the file descriptor */
-  output ("static cob_file %s%s = {", CB_PREFIX_FILE, f->cname);
+  output ("  static cob_file %s%s = {", CB_PREFIX_FILE, f->cname);
   /* organization, access_mode, open_mode, flag_optional */
   output ("%d, %d, 0, %d, ", f->organization, f->access_mode, f->optional);
   /* file_status */
@@ -1842,6 +1938,35 @@ output_file_definition (struct cb_file *f)
     output ("0, 0, ");
   /* file */
     output ("0, ");
+  /* LINAGE */
+  if ( f->linage ) {
+	output_param (f->linage, -1);
+	output (", ");
+	output_param (f->linage_ctr, -1);
+	output (", ");
+	if ( f->latfoot ) {
+		output_param (f->latfoot, -1);
+	} else {
+		output ("NULL");
+	}
+	output (", ");
+	if ( f->lattop ) {
+		output_param (f->lattop, -1);
+	} else {
+		output ("NULL");
+	}
+	output (", ");
+	if ( f->lattop ) {
+		output_param (f->latbot, -1);
+	} else {
+		output ("NULL");
+	}
+	output (", ");
+  } else {
+	output ("NULL, NULL, NULL, NULL, NULL, ");
+  }
+  /* LINAGE  Current values */
+  output ("0, 0, 0, 0, ");
   /* flags */
   output ("0, 0, 0, 0, 0, ");
   /* has file status flag */
@@ -1849,11 +1974,11 @@ output_file_definition (struct cb_file *f)
     output ("1, ");
   else
     output ("0, ");
-  /* spare bytes */
+  /* LS close needs NL / Linage needs top */
   output ("0, 0};\n\n");
 }
 
-
+
 /*
  * Screen definition
  */
@@ -1911,7 +2036,7 @@ output_screen_definition (struct cb_field *p)
   output (", %ld};\n", p->screen_flag);
 }
 
-
+
 /*
  * Alphabet-name
  */
@@ -1987,7 +2112,7 @@ output_alphabet_name_definition (struct cb_alphabet_name *p)
   output ("};\n\n");
 }
 
-
+
 /*
  * Class definition
  */
@@ -2035,13 +2160,15 @@ output_class_name_definition (struct cb_class_name *p)
   output_newline ();
 }
 
-
+
 static void
 output_initial_values (struct cb_field *p)
 {
   cb_tree def = cb_auto_initialize ? cb_true : NULL;
   for (; p; p = p->sister)
+/*
     if (!p->flag_external)
+*/
       {
 	cb_tree x = cb_build_field_reference (p, 0);
 	output_stmt (cb_build_initialize (x, cb_true, NULL, def));
@@ -2058,31 +2185,63 @@ output_internal_function (struct cb_program *prog, cb_tree parameter_list)
   /* program function */
   output ("static int\n%s_ (int entry", prog->program_id);
   for (l = parameter_list; l; l = CB_CHAIN (l))
-    output (", unsigned char *%s%d",
-	    CB_PREFIX_BASE, cb_field (CB_VALUE (l))->id);
+	output (", unsigned char *%s%d",
+		CB_PREFIX_BASE, cb_field (CB_VALUE (l))->id);
   output (")\n");
   output_indent ("{");
 
   /* local variables */
   output_line ("static int initialized = 0;");
-  output_line ("static cob_decimal d[%d];", prog->decimal_index_max);
+  if (prog->decimal_index_max)
+	output_line ("static cob_decimal d[%d];", prog->decimal_index_max);
+
   output_prefix ();
-  output ("static cob_module module = {'%c', '%c', '%c', ",
-	  prog->decimal_point, prog->currency_symbol, prog->numeric_separator);
+  output ("static cob_module module = { NULL, ");
   if (prog->collating_sequence)
-    output_param (cb_ref (prog->collating_sequence), -1);
-  else
-    output ("0");
-  output (", %d, %d, %d, %d, 0};\n",
-	  cb_display_sign, cb_filename_mapping, cb_binary_truncate,
-	  cb_pretty_display);
+	output_param (cb_ref (prog->collating_sequence), -1);
+  else    
+	output ("NULL");
+  output (", %d, '%c', '%c', '%c', %d, %d, %d};\n",
+	cb_display_sign, prog->decimal_point, prog->currency_symbol,
+	prog->numeric_separator, cb_filename_mapping, cb_binary_truncate,
+	cb_pretty_display);
   output_newline ();
-  output_line ("int i;");
-  output_line ("int i1, i2, i3, i4, i5, i6, i7;");
-  output_line ("int n[%d];", prog->loop_counter);
-  output_line ("int frame_index;");
-  output_line ("struct frame { int perform_through; void *return_address; } "
-	       "frame_stack[254];");
+
+  /* External items */
+  for (f = prog->working_storage; f; f = f->sister) {
+	if ( f->flag_external ) {
+		char name[CB_MAX_CNAME];
+		char *p;
+
+		strcpy (name, f->name);
+		for (p = name; *p; p++) {
+			if (*p == '-')
+				*p = '_';
+		}
+		output ("  static unsigned char *%s%s = NULL;",
+			CB_PREFIX_BASE, name);
+		output ("  /* %s */\n", f->name);
+	}
+  }
+  output_newline ();
+
+  /* files */
+  if (prog->file_list)
+    {
+      output ("  /* Files */\n\n");
+      for (l = prog->file_list; l; l = CB_CHAIN (l))
+	output_file_definition (CB_FILE (CB_VALUE (l)));
+      output_newline ();
+    }
+
+  for ( i = 0; i < 8; i++ ) {
+	if ( i_counters[i] ) {
+		output_line ("int i%d;", i);
+	}
+  }
+
+  if ( prog->loop_counter )
+	output_line ("int n[%d];", prog->loop_counter);
   output_line ("cob_field f[4];");
   output_newline ();
 
@@ -2097,15 +2256,6 @@ output_internal_function (struct cb_program *prog, cb_tree parameter_list)
     }
   output_newline ();
 
-  /* files */
-  if (prog->file_list)
-    {
-      output ("/* Files */\n\n");
-      for (l = prog->file_list; l; l = CB_CHAIN (l))
-	output_file_definition (CB_FILE (CB_VALUE (l)));
-      output_newline ();
-    }
-
   /* screens */
   if (prog->screen_storage)
     {
@@ -2115,19 +2265,38 @@ output_internal_function (struct cb_program *prog, cb_tree parameter_list)
       output_newline ();
     }
 
+  output_line ("/* perform frame stack */");
+  output_line ("int frame_index;");
+  output_line ("struct frame { int perform_through; void *return_address; } "
+	       "frame_stack[254];");
+  output_newline ();
+
+  output_line ("/* Start of function code */");
+  output_newline ();
   output_line ("cob_module_enter (&module);");
   output_newline ();
 
   /* initialization */
+#if defined (__GNUC__) && (__GNUC__ >= 3)
+  output_line ("if ( __builtin_expect(!initialized, 0) )");
+#else
   output_line ("if (!initialized)");
+#endif
   output_indent ("  {");
-  output_stmt (cb_build_assign (cb_return_code, cb_int0));
-  output_line ("/* initialize decimal numbers */");
-  output_line ("for (i = 0; i < %d; i++)", prog->decimal_index_max);
-  output_line ("  cob_decimal_init (&d[i]);");
-  output_newline ();
-  if (!prog->flag_initial)
+  /* output_stmt (cb_build_assign (cb_return_code, cb_int0)); */
+  if (prog->decimal_index_max) {
+	output_line ("/* initialize decimal numbers */");
+	output_line ("{ int i;");
+	output_line ("for (i = 0; i < %d; i++)", prog->decimal_index_max);
+	output_line ("  cob_decimal_init (&d[i]);");
+	output_line ("}");
+	output_newline ();
+  }
+  if (!prog->flag_initial) {
     output_initial_values (prog->working_storage);
+    output_newline ();
+  }
+
   output_line ("initialized = 1;");
   output_indent ("  }");
   if (prog->flag_initial)
@@ -2136,22 +2305,50 @@ output_internal_function (struct cb_program *prog, cb_tree parameter_list)
   output_newline ();
 
   output_line ("/* initialize frame stack */");
-  output_line ("frame_index = 0;");
+  output_line ("frame_index = -1;");
   output_line ("frame_stack[0].perform_through = -1;");
   output_newline ();
 
-  /* entry dispatch */
-  output_line ("switch (entry)");
-  output_line ("  {");
-  for (i = 0, l = prog->entry_list; l; l = CB_CHAIN (l))
-    {
-      output_line ("  case %d:", i++);
-      output_line ("    goto %s%d;",
-		   CB_PREFIX_LABEL, CB_LABEL (CB_PURPOSE (l))->id);
-    }
-  output_line ("  }");
+  output_line ("/* initialize number of call params */");
+  output ("  ");
+  output_integer (cb_call_params);
+  output_line (" = cob_call_params;");
   output_newline ();
 
+  /* entry dispatch */
+  if ( entry_number > 1 ) {
+	output_line ("switch (entry)");
+	output_line ("  {");
+	for (i = 0, l = prog->entry_list; l; l = CB_CHAIN (l))
+	  {
+	    output_line ("  case %d:", i++);
+	    output_line ("    goto %s%d;",
+			 CB_PREFIX_LABEL, CB_LABEL (CB_PURPOSE (l))->id);
+	  }
+	output_line ("  }");
+	output_newline ();
+  } else {
+	l = prog->entry_list;
+	output_line ("goto %s%d;",
+		CB_PREFIX_LABEL, CB_LABEL (CB_PURPOSE (l))->id);
+	output_newline ();
+  }
+
+  /* PROCEDURE DIVISION */
+  output_line ("/* PROCEDURE DIVISION */");
+  for (l = prog->exec_list; l; l = CB_CHAIN (l))
+    output_stmt (CB_VALUE (l));
+  output_newline ();
+
+  if ( needs_exit_prog ) {
+	output_line ("exit_program:");
+  }
+  output_line ("cob_module_leave (&module);");
+  output_prefix ();
+  output ("return ");
+  output_integer (cb_return_code);
+  output (";\n\n");
+
   /* error handlers */
   output_line ("/* error handlers */");
   output_stmt (cb_standard_error_handler);
@@ -2173,20 +2370,10 @@ output_internal_function (struct cb_program *prog, cb_tree parameter_list)
   output_line ("    break;");
   output_line ("  }");
   output_perform_exit (CB_LABEL (cb_standard_error_handler));
+  output_line ("fprintf(stderr, \"Codegen error\\n\");");
+  output_line ("exit(1);");
   output_newline ();
 
-  /* PROCEDURE DIVISION */
-  output_line ("/* PROCEDURE DIVISION */");
-  for (l = prog->exec_list; l; l = CB_CHAIN (l))
-    output_stmt (CB_VALUE (l));
-  output_newline ();
-
-  output_line ("exit_program:");
-  output_line ("cob_module_leave (&module);");
-  output_prefix ();
-  output ("return ");
-  output_integer (cb_return_code);
-  output (";\n");
   output_indent ("}");
   output_newline ();
 }
@@ -2196,7 +2383,6 @@ output_entry_function (struct cb_program *prog,
 		       cb_tree entry,
 		       cb_tree parameter_list)
 {
-  static int id = 0;
 
   const char *entry_name = CB_LABEL (CB_PURPOSE (entry))->name;
   cb_tree using_list = CB_VALUE (entry);
@@ -2239,7 +2425,7 @@ output_entry_function (struct cb_program *prog,
 		     f->id, CB_PREFIX_BASE, f->id, f->size);
       }
 
-  output ("  return %s_ (%d", prog->program_id, id++);
+  output ("  return %s_ (%d", prog->program_id, progid++);
   for (l1 = parameter_list; l1; l1 = CB_CHAIN (l1))
     {
       for (l2 = using_list; l2; l2 = CB_CHAIN (l2))
@@ -2282,7 +2468,7 @@ output_main_function (struct cb_program *prog)
   if (prog->flag_screen)
     output_line ("cob_screen_clear ();");
   output_line ("return ret;");
-  output_indent ("}");
+  output_indent ("}\n");
 }
 
 void
@@ -2290,6 +2476,19 @@ codegen (struct cb_program *prog)
 {
   cb_tree l;
   cb_tree parameter_list = NULL;
+  struct attr_list *j;
+  struct literal_list *m;
+  struct field_list *k;
+
+  param_id = 0;
+  progid = 0;
+  loop_counter = 0;
+  output_indent_level = 0;
+  last_line = 0;
+  needs_exit_prog = 0;
+  field_cache = NULL;
+  attr_cache = NULL;
+  literal_cache = NULL;
 
   if (cb_flag_main)
     prog->flag_initial = 1;
@@ -2330,12 +2529,50 @@ codegen (struct cb_program *prog)
 	}
     }
 
-  /* functions */
-  output_internal_function (prog, parameter_list);
-  for (l = prog->entry_list; l; l = CB_CHAIN (l))
-    output_entry_function (prog, l, parameter_list);
+  /* prototype */
+  output ("static int %s_ (int", prog->program_id);
+  for (l = parameter_list; l; l = CB_CHAIN (l))
+	output (", unsigned char *");
+  output (");\n\n");
 
   /* main function */
   if (cb_flag_main)
     output_main_function (prog);
+
+  /* functions */
+  for (l = prog->entry_list; l; l = CB_CHAIN (l))
+    output_entry_function (prog, l, parameter_list);
+  output_internal_function (prog, parameter_list);
+
+  output_target = cb_storage_file;
+  for (j = attr_cache; j; j = j->next) {
+	output_storage ("static cob_field_attr %s%d\t= ", CB_PREFIX_ATTR, j->id);
+	output_storage ("{%d, %d, %d, %d, ", j->type, j->digits, j->scale, j->flags);
+	if (j->pic)
+	    {
+	      unsigned char *s;
+	      output_storage ("\"");
+	      for (s = j->pic; *s; s += 2)
+		output_storage ("%c\\%03o", s[0], s[1]);
+	      output_storage ("\"");
+	    }
+	else
+	output_storage ("0");
+	output_storage ("};\n");
+  }
+
+  for (k = field_cache; k; k = k->next) {
+	output ("static cob_field %s%d\t= ", CB_PREFIX_FIELD, k->f->id);
+	output_field (k->x);
+	output (";\t/* %s */\n", k->f->name);
+  }
+
+  for (m = literal_cache; m; m = m->next) {
+	output ("static cob_field %s%d\t= ", CB_PREFIX_CONST, m->id);
+	output_field (m->x);
+	output (";\n");
+  }
+
+  output_target = yyout;
+
 }
