@@ -87,6 +87,8 @@ static struct reserved reserved_words[] = {
   {"ARE",		ARE, 0},			/* 2002 */
   {"AREA",		AREA, 0},			/* 2002 */
   {"AREAS",		AREA, 0},			/* 2002 */
+  {"ARGUMENT-NUMBER",	ARGUMENT_NUMBER, 0},		/* extension */
+  {"ARGUMENT-VALUE",	ARGUMENT_VALUE, 0},		/* extension */
   {"AS",		AS, 0},				/* 2002 */
   {"ASCENDING",		ASCENDING, 0},			/* 2002 */
   {"ASSIGN",		ASSIGN, 0},			/* 2002 */
@@ -125,7 +127,6 @@ static struct reserved reserved_words[] = {
   {"CLOSE",		CLOSE, 0},			/* 2002 */
   {"CODE",		-1, 0},				/* 2002 */
   {"CODE-SET",		CODE_SET, 0},			/* 2002 */
-  {"CODITION",		-1, 0},				/* 2002 */
   {"COL",		COLUMN, 0},			/* 2002 */
   {"COLLATING",		COLLATING, 0},			/* 2002 */
   {"COLS",		COLUMN, 0},			/* 2002 */
@@ -150,6 +151,7 @@ static struct reserved reserved_words[] = {
   {"COMPUTATIONAL-5",	COMP_5, 0},			/* extension */
   {"COMPUTATIONAL-X",	COMP_X, 0},			/* extension */
   {"COMPUTE",		COMPUTE, 0},			/* 2002 */
+  {"CONDITION",		-1, 0},				/* 2002 */
   {"CONFIGURATION",	CONFIGURATION, 0},		/* 2002 */
   {"CONSTANT",		-1, 0},				/* 2002 */
   {"CONTAINS",		CONTAINS, 0},			/* 2002 */
@@ -305,7 +307,7 @@ static struct reserved reserved_words[] = {
   {"LIMIT",		-1, 0},				/* 2002 */
   {"LIMITS",		-1, 0},				/* 2002 */
   {"LINAGE",		LINAGE, 0},			/* 2002 */
-  {"LINAGE-COUNTER",	0, 0},				/* 2002 */
+  {"LINAGE-COUNTER",	LINAGE_COUNTER, 0},		/* 2002 */
   {"LINE",		LINE, 0},			/* 2002 */
   {"LINE-COUNTER",	-1, 0},				/* 2002 */
   {"LINES",		LINES, 0},			/* 2002 */
@@ -519,7 +521,7 @@ static struct reserved reserved_words[] = {
   {0, 0, 0}
 };
 
-
+
 static int
 hash (const char *s)
 {
@@ -546,8 +548,16 @@ lookup_reserved_word (const char *name)
   for (p = reserved_table[hash (name)]; p; p = p->next)
     if (strcasecmp (name, p->name) == 0)
       {
-	if (p->token != -1)
+	if (p->token != -1) {
+	  struct noreserve *noresptr;
+
+	  for ( noresptr = norestab; noresptr; noresptr = noresptr->next) {
+		if (strcasecmp (name, noresptr->noresword) == 0) {
+			return 0;
+		}
+	  }
 	  return p->token;
+	}
 	cb_error (_("`%s' reserved word, but not supported yet"), name);
 	return 0;
       }
