@@ -40,7 +40,7 @@ to_cname (const char *s)
   return copy;
 }
 
-
+
 /*
  * Tree
  */
@@ -294,6 +294,7 @@ cb_tree_type (cb_tree x)
 	case CB_USAGE_COMP_5:
 	case CB_USAGE_COMP_X:
 	case CB_USAGE_INDEX:
+	case CB_USAGE_LENGTH:
 	  return COB_TYPE_NUMERIC_BINARY;
 	case CB_USAGE_PACKED:
 	  return COB_TYPE_NUMERIC_PACKED;
@@ -329,6 +330,7 @@ cb_fits_int (cb_tree x)
 	switch (f->usage)
 	  {
 	  case CB_USAGE_INDEX:
+	  case CB_USAGE_LENGTH:
 	    return 1;
 	  case CB_USAGE_BINARY:
 	  case CB_USAGE_COMP_5:
@@ -374,7 +376,7 @@ cb_get_int (cb_tree x)
   return val;
 }
 
-
+
 /*
  * Constants
  */
@@ -391,9 +393,11 @@ cb_tree cb_quote;
 cb_tree cb_int0;
 cb_tree cb_int1;
 cb_tree cb_int2;
+cb_tree cb_int3;
 cb_tree cb_i[8];
 cb_tree cb_error_node;
 cb_tree cb_return_code;
+cb_tree cb_call_params;
 cb_tree cb_standard_error_handler;
 
 static cb_tree
@@ -430,6 +434,7 @@ cb_init_constants (void)
   cb_int0        = cb_int (0);
   cb_int1        = cb_int (1);
   cb_int2        = cb_int (2);
+  cb_int3        = cb_int (3);
   for (i = 1; i < 8; i++)
     {
       char *s = malloc (3);
@@ -439,7 +444,7 @@ cb_init_constants (void)
   cb_standard_error_handler = make_constant_label ("standard_error_handler");
 }
 
-
+
 /*
  * Integer
  */
@@ -473,7 +478,7 @@ cb_int (int n)
   return p->node;
 }
 
-
+
 /*
  * String
  */
@@ -488,7 +493,7 @@ cb_build_string (const unsigned char *data, size_t size)
   return CB_TREE (p);
 }
 
-
+
 /*
  * Alphabet-name
  */
@@ -504,7 +509,7 @@ cb_build_alphabet_name (cb_tree name, enum cb_alphabet_name_type type)
   return CB_TREE (p);
 }
 
-
+
 /*
  * Class-name
  */
@@ -522,7 +527,7 @@ cb_build_class_name (cb_tree name, cb_tree list)
   return CB_TREE (p);
 }
 
-
+
 /*
  * System-name
  */
@@ -537,7 +542,7 @@ cb_build_system_name (enum cb_system_name_category category, int token)
   return CB_TREE (p);
 }
 
-
+
 /*
  * Literal
  */
@@ -547,9 +552,9 @@ build_literal (enum cb_category category, const unsigned char *data, size_t size
 {
   struct cb_literal *p =
     make_tree (CB_TAG_LITERAL, category, sizeof (struct cb_literal));
-  p->data = malloc (size + 1);
+  p->data = malloc ((size_t)(size + 1));
   p->size = size;
-  memcpy (p->data, data, size);
+  memcpy (p->data, data, (size_t)size);
   p->data[size] = 0;
   return p;
 }
@@ -581,7 +586,7 @@ cb_concat_literals (cb_tree x1, cb_tree x2)
   return cb_build_alphanumeric_literal (buff, l1->size + l2->size);
 }
 
-
+
 /*
  * Decimal
  */
@@ -595,7 +600,7 @@ cb_build_decimal (int id)
   return CB_TREE (p);
 }
 
-
+
 /*
  * Picture
  */
@@ -839,7 +844,7 @@ cb_build_picture (const char *str)
   return CB_TREE (pic);
 }
 
-
+
 /*
  * Field
  */
@@ -851,6 +856,7 @@ cb_build_field (cb_tree name)
     make_tree (CB_TAG_FIELD, CB_CATEGORY_UNKNOWN, sizeof (struct cb_field));
   p->id = cb_id++;
   p->name = cb_define (name, CB_TREE (p));
+  p->ename = NULL;
   p->usage = CB_USAGE_DISPLAY;
   p->storage = CB_STORAGE_WORKING;
   p->occurs_max = 1;
@@ -983,7 +989,7 @@ cb_field_subordinate (struct cb_field *p, struct cb_field *f)
   return 0;
 }
 
-
+
 /*
  * File
  */
@@ -1061,6 +1067,20 @@ finalize_file (struct cb_file *f, struct cb_field *records)
   sprintf (buff, "%s$record", f->name);
   f->record = CB_FIELD (cb_build_implicit_field (cb_build_reference (buff),
 						 f->record_max));
+  if ( f->linage ) {
+	cb_tree x;
+
+	sprintf (buff, "LC$%s", f->name);
+	x = cb_build_field (cb_build_reference (buff));
+	CB_FIELD (x)->pic = CB_PICTURE (cb_build_picture ("9(9)"));
+	CB_FIELD (x)->usage = CB_USAGE_COMP_5;
+	CB_FIELD (x)->values = cb_list (cb_zero);
+	CB_FIELD (x)->count++;
+	cb_validate_field (CB_FIELD (x));
+	f->linage_ctr = x;
+	current_program->working_storage =
+		cb_field_add (current_program->working_storage, CB_FIELD (x));
+  }
   f->record->sister = records;
   f->record->count++;
 
@@ -1071,7 +1091,7 @@ finalize_file (struct cb_file *f, struct cb_field *records)
     }
 }
 
-
+
 /*
  * Reference
  */
@@ -1131,6 +1151,7 @@ cb_tree
 cb_ref (cb_tree x)
 {
   struct cb_reference *r = CB_REFERENCE (x);
+  int ambiguous = 0;
   cb_tree candidate = NULL;
   cb_tree items;
 
@@ -1203,9 +1224,10 @@ cb_ref (cb_tree x)
 	    }
 	  else
 	    {
-	      /* there are several candidates */
-	      ambiguous_error (x);
-	      goto error;
+	      /* there are several candidates and possibly ambiguous */
+	      ambiguous = 1;
+	      /* continue search because the reference might not ambiguous
+		and exit loop by "goto end" later on */
 	    }
 	}
     }
@@ -1217,6 +1239,13 @@ cb_ref (cb_tree x)
       goto error;
     }
 
+  /* the reference is ambiguous */
+  if (ambiguous)
+    {
+      ambiguous_error (x);
+      goto error;
+    }
+
  end:
   if (CB_FIELD_P (candidate))
     {
@@ -1233,7 +1262,7 @@ cb_ref (cb_tree x)
   return cb_error_node;
 }
 
-
+
 /*
  * Expression
  */
@@ -1299,7 +1328,7 @@ cb_build_binary_list (cb_tree l, char op)
   return e;
 }
 
-
+
 /*
  * Function call
  */
@@ -1319,7 +1348,7 @@ cb_build_funcall (const char *name, int argc,
   return CB_TREE (p);
 }
 
-
+
 /*
  * Type cast
  */
@@ -1336,7 +1365,7 @@ cb_build_cast (enum cb_cast_type type, cb_tree val)
   return CB_TREE (p);
 }
 
-
+
 /*
  * Label
  */
@@ -1352,7 +1381,7 @@ cb_build_label (cb_tree name, struct cb_label *section)
   return CB_TREE (p);
 }
 
-
+
 /*
  * Assign
  */
@@ -1367,7 +1396,7 @@ cb_build_assign (cb_tree var, cb_tree val)
   return CB_TREE (p);
 }
 
-
+
 /*
  * INITIALIZE
  */
@@ -1384,7 +1413,7 @@ cb_build_initialize (cb_tree var, cb_tree val, cb_tree rep, cb_tree def)
   return CB_TREE (p);
 }
 
-
+
 /*
  * SEARCH
  */
@@ -1403,7 +1432,7 @@ cb_build_search (int flag_all, cb_tree table, cb_tree var,
   return CB_TREE (p);
 }
 
-
+
 /*
  * CALL
  */
@@ -1420,7 +1449,7 @@ cb_build_call (cb_tree name, cb_tree args, cb_tree stmt1, cb_tree stmt2)
   return CB_TREE (p);
 }
 
-
+
 /*
  * GO TO
  */
@@ -1435,7 +1464,7 @@ cb_build_goto (cb_tree target, cb_tree depending)
   return CB_TREE (p);
 }
 
-
+
 /*
  * IF
  */
@@ -1451,7 +1480,7 @@ cb_build_if (cb_tree test, cb_tree stmt1, cb_tree stmt2)
   return CB_TREE (p);
 }
 
-
+
 /*
  * PERFORM
  */
@@ -1477,7 +1506,7 @@ cb_build_perform_varying (cb_tree name, cb_tree from, cb_tree by, cb_tree until)
   return CB_TREE (p);
 }
 
-
+
 /*
  * Statement
  */
@@ -1491,7 +1520,7 @@ cb_build_statement (const char *name)
   return p;
 }
 
-
+
 /*
  * List
  */
@@ -1559,7 +1588,7 @@ cb_list_map (cb_tree (*func) (cb_tree x), cb_tree l)
     CB_VALUE (l) = func (CB_VALUE (l));
 }
 
-
+
 /*
  * Program
  */
