@@ -161,6 +161,7 @@ enum cb_usage {
   CB_USAGE_PACKED,
   CB_USAGE_POINTER,
   CB_USAGE_PROGRAM,
+  CB_USAGE_LENGTH
 };
 
 enum cb_operand_type {
@@ -168,7 +169,7 @@ enum cb_operand_type {
   CB_RECEIVING_OPERAND
 };
 
-
+
 /*
  * Tree
  */
@@ -212,7 +213,7 @@ extern int cb_tree_type (cb_tree x);
 extern int cb_fits_int (cb_tree x);
 extern int cb_get_int (cb_tree x);
 
-
+
 /*
  * Constants
  */
@@ -229,9 +230,11 @@ extern cb_tree cb_quote;
 extern cb_tree cb_int0;
 extern cb_tree cb_int1;
 extern cb_tree cb_int2;
+extern cb_tree cb_int3;
 extern cb_tree cb_i[8];
 extern cb_tree cb_error_node;
 extern cb_tree cb_return_code;
+extern cb_tree cb_call_params;
 extern cb_tree cb_standard_error_handler;
 
 struct cb_const {
@@ -244,7 +247,7 @@ struct cb_const {
 
 extern void cb_init_constants (void);
 
-
+
 /*
  * Integer
  */
@@ -259,7 +262,7 @@ struct cb_integer {
 
 extern cb_tree cb_int (int n);
 
-
+
 /*
  * String
  */
@@ -277,7 +280,7 @@ struct cb_string {
 
 extern cb_tree cb_build_string (const unsigned char *data, size_t size);
 
-
+
 /*
  * Alphabet-name
  */
@@ -295,7 +298,7 @@ struct cb_alphabet_name {
 
 extern cb_tree cb_build_alphabet_name (cb_tree name, enum cb_alphabet_name_type type);
 
-
+
 /*
  * Class-name
  */
@@ -312,7 +315,7 @@ struct cb_class_name {
 
 extern cb_tree cb_build_class_name (cb_tree name, cb_tree list);
 
-
+
 /*
  * System-name
  */
@@ -328,7 +331,7 @@ struct cb_system_name {
 
 extern cb_tree cb_build_system_name (enum cb_system_name_category category, int token);
 
-
+
 /*
  * Literal
  */
@@ -340,6 +343,7 @@ struct cb_literal {
   char all;
   char sign;			/* unsigned: 0 negative: -1 positive: 1 */
   char scale;
+  char spare;			/* spare */
 };
 
 #define CB_LITERAL(x)	(CB_TREE_CAST (CB_TAG_LITERAL, struct cb_literal, x))
@@ -351,7 +355,7 @@ extern cb_tree cb_build_numeric_literal (int sign, const unsigned char *data, in
 extern cb_tree cb_build_alphanumeric_literal (const unsigned char *data, size_t size);
 extern cb_tree cb_concat_literals (cb_tree x1, cb_tree x2);
 
-
+
 /*
  * Decimal
  */
@@ -366,7 +370,7 @@ struct cb_decimal {
 
 extern cb_tree cb_build_decimal (int id);
 
-
+
 /*
  * Picture
  */
@@ -380,6 +384,7 @@ struct cb_picture {
   char digits;			/* the number of digit places */
   char scale;			/* 1/10^scale */
   char have_sign;		/* have 'S' */
+  char spare;			/* spare */
 };
 
 #define CB_PICTURE(x)	(CB_TREE_CAST (CB_TAG_PICTURE, struct cb_picture, x))
@@ -387,7 +392,7 @@ struct cb_picture {
 
 extern cb_tree cb_build_picture (const char *str);
 
-
+
 /*
  * Field
  */
@@ -396,6 +401,7 @@ struct cb_field {
   struct cb_tree_common common;
   int id;			/* field id */
   const char *name;		/* the original name */
+  const char *ename;		/* the externalized name */
   int size;			/* field size */
   int memory_size;		/* memory size */
   int offset;			/* byte offset from the top (ie, 01 field) */
@@ -423,25 +429,26 @@ struct cb_field {
   } *keys;
   int nkeys;			/* the number of keys */
   struct cb_picture *pic;	/* PICTURE */
-  /* flags */
-  long flag_external      : 1;	/* EXTERNAL */
-  long flag_blank_zero    : 1;	/* BLANK WHEN ZERO */
-  long flag_justified     : 1;	/* JUSTIFIED RIGHT */
-  long flag_sign_leading  : 1;	/* SIGN IS LEADING */
-  long flag_sign_separate : 1;	/* SIGN IS SEPARATE */
-  long flag_synchronized  : 1;	/* SYNCHRONIZED */
-  long flag_occurs        : 1;	/* OCCURS */
-  long flag_invalid       : 1;	/* is broken */
-  long flag_binary_swap   : 1;	/* binary byteswap */
-  long flag_local         : 1;	/* has local scope */
-  long flag_base          : 1;
-  long flag_field         : 1;
   /* screen parameters */
   cb_tree screen_line;
   cb_tree screen_column;
   cb_tree screen_from;
   cb_tree screen_to;
   long screen_flag;		/* flags used in SCREEN SECTION */
+  /* flags */
+  unsigned int flag_external      : 1;	/* EXTERNAL */
+  unsigned int flag_blank_zero    : 1;	/* BLANK WHEN ZERO */
+  unsigned int flag_justified     : 1;	/* JUSTIFIED RIGHT */
+  unsigned int flag_sign_leading  : 1;	/* SIGN IS LEADING */
+  unsigned int flag_sign_separate : 1;	/* SIGN IS SEPARATE */
+  unsigned int flag_synchronized  : 1;	/* SYNCHRONIZED */
+  unsigned int flag_occurs        : 1;	/* OCCURS */
+  unsigned int flag_invalid       : 1;	/* is broken */
+  unsigned int flag_binary_swap   : 1;	/* binary byteswap */
+  unsigned int flag_local         : 1;	/* has local scope */
+  unsigned int flag_base          : 1;
+  unsigned int flag_field         : 1;
+  unsigned int flag_spare	 : 20;
 };
 
 #define CB_FIELD(x)		(CB_TREE_CAST (CB_TAG_FIELD, struct cb_field, x))
@@ -465,7 +472,7 @@ extern int cb_field_subordinate (struct cb_field *p, struct cb_field *f);
   ((CB_FIELD_P (x) || CB_REFERENCE_P (x))	\
    && cb_field (x)->usage == CB_USAGE_INDEX)
 
-
+
 /*
  * File
  */
@@ -491,6 +498,11 @@ struct cb_file {
   struct cb_field *record;	/* record descriptor */
   int record_min, record_max;	/* RECORD CONTAINS */
   cb_tree record_depending;	/* RECORD DEPENDING */
+  cb_tree linage_ctr;
+  cb_tree linage;
+  cb_tree latfoot;
+  cb_tree lattop;
+  cb_tree latbot;
   /* STANDARD ERROR PROCEDURE */
   struct cb_label *handler;	/* error handler */
 };
@@ -502,7 +514,7 @@ extern struct cb_file *build_file (cb_tree name);
 extern void validate_file (struct cb_file *f, cb_tree name);
 extern void finalize_file (struct cb_file *f, struct cb_field *records);
 
-
+
 /*
  * Reference
  */
@@ -531,7 +543,7 @@ extern const char *cb_define (cb_tree name, cb_tree val);
 extern void cb_define_system_name (const char *name);
 extern cb_tree cb_ref (cb_tree x);
 
-
+
 /*
  * Binary operation
  */
@@ -570,7 +582,7 @@ struct cb_binary_op {
 extern cb_tree cb_build_binary_op (cb_tree x, char op, cb_tree y);
 extern cb_tree cb_build_binary_list (cb_tree l, char op);
 
-
+
 /*
  * Function call
  */
@@ -593,7 +605,7 @@ extern cb_tree cb_build_funcall (const char *name, int argc, cb_tree a1, cb_tree
 #define cb_build_funcall_3(f,a1,a2,a3)	cb_build_funcall (f, 3, a1, a2, a3, 0)
 #define cb_build_funcall_4(f,a1,a2,a3,a4) cb_build_funcall (f, 4, a1, a2, a3, a4)
 
-
+
 /*
  * Type cast
  */
@@ -619,7 +631,7 @@ extern cb_tree cb_build_cast (enum cb_cast_type type, cb_tree val);
 #define cb_build_cast_address(x)	cb_build_cast (CB_CAST_ADDRESS, x)
 #define cb_build_cast_length(x)		cb_build_cast (CB_CAST_LENGTH, x)
 
-
+
 /*
  * Label
  */
@@ -639,7 +651,7 @@ struct cb_label {
 
 extern cb_tree cb_build_label (cb_tree name, struct cb_label *section);
 
-
+
 /*
  * Assign
  */
@@ -655,7 +667,7 @@ struct cb_assign {
 
 extern cb_tree cb_build_assign (cb_tree var, cb_tree val);
 
-
+
 /*
  * INITIALIZE
  */
@@ -673,7 +685,7 @@ struct cb_initialize {
 
 extern cb_tree cb_build_initialize (cb_tree var, cb_tree val, cb_tree rep, cb_tree def);
 
-
+
 /*
  * SEARCH
  */
@@ -692,7 +704,7 @@ struct cb_search {
 
 extern cb_tree cb_build_search (int flag_all, cb_tree table, cb_tree var, cb_tree end_stmt, cb_tree whens);
 
-
+
 /*
  * CALL
  */
@@ -714,7 +726,7 @@ struct cb_call {
 
 extern cb_tree cb_build_call (cb_tree name, cb_tree args, cb_tree stmt1, cb_tree stmt2);
 
-
+
 /*
  * GO TO statement
  */
@@ -730,7 +742,7 @@ struct cb_goto {
 
 extern cb_tree cb_build_goto (cb_tree target, cb_tree depending);
 
-
+
 /*
  * IF
  */
@@ -747,7 +759,7 @@ struct cb_if {
 
 extern cb_tree cb_build_if (cb_tree test, cb_tree stmt1, cb_tree stmt2);
 
-
+
 /*
  * PERFORM
  */
@@ -806,7 +818,7 @@ struct cb_statement {
 
 extern struct cb_statement *cb_build_statement (const char *name);
 
-
+
 /*
  * List
  */
@@ -844,7 +856,7 @@ extern int cb_list_length (cb_tree l);
 
 #define cb_build_pair(x,y)	cb_build_list (x, y, 0)
 
-
+
 /*
  * Program
  */
@@ -865,6 +877,7 @@ struct cb_program {
   unsigned char decimal_point;		/* '.' or ',' */
   unsigned char currency_symbol;	/* '$' or user-specified */
   unsigned char numeric_separator;	/* ',' or '.' */
+  unsigned char spare;			/* spare */
   cb_tree entry_list;
   cb_tree file_list;
   cb_tree exec_list;
@@ -878,10 +891,10 @@ struct cb_program {
   struct cb_field *screen_storage;
   struct cb_label *file_handler[5];
   cb_tree collating_sequence;
-  long flag_common      : 1;		/* COMMON PROGRAM */
-  long flag_initial     : 1;		/* INITIAL PROGRAM */
-  long flag_recursive   : 1;		/* RECURSIVE PROGRAM */
-  long flag_screen      : 1;		/* have SCREEN SECTION */
+  char flag_common;		/* COMMON PROGRAM */
+  char flag_initial;		/* INITIAL PROGRAM */
+  char flag_recursive;		/* RECURSIVE PROGRAM */
+  char flag_screen;		/* have SCREEN SECTION */
   /* internal variables */
   int loop_counter;
   int decimal_index;
@@ -891,7 +904,7 @@ struct cb_program {
 
 extern struct cb_program *cb_build_program (void);
 
-
+
 /* reserved.c */
 extern cb_tree lookup_system_name (const char *name);
 extern int lookup_reserved_word (const char *name);
@@ -965,6 +978,8 @@ extern void cb_emit_accept_command_line (cb_tree var);
 extern void cb_emit_accept_environment (cb_tree var);
 extern void cb_emit_accept_mnemonic (cb_tree var, cb_tree mnemonic);
 extern void cb_emit_accept_name (cb_tree var, cb_tree name);
+extern void cb_emit_accept_arg_number (cb_tree var);
+extern void cb_emit_accept_arg_value (cb_tree var);
 
 extern void cb_emit_call (cb_tree prog, cb_tree using, cb_tree returning, cb_tree on_exception, cb_tree not_on_exception);
 
