@@ -44,7 +44,7 @@
 	return;					\
   }
 
-
+
 cb_tree
 cb_check_group_name (cb_tree x)
 {
@@ -148,19 +148,24 @@ cb_check_integer_value (cb_tree x)
     }
 }
 
-
+
 void
 cb_build_registers (void)
 {
   /* RETURN-CODE */
   cb_return_code = cb_build_index (cb_build_reference ("RETURN-CODE"));
+  CB_FIELD (cb_ref (cb_return_code))->values = cb_list (cb_zero);
+
+  /* NUMBER-OF-CALL-PARAMETERS */
+  cb_call_params = cb_build_index (cb_build_reference ("NUMBER-OF-CALL-PARAMETERS"));
+  CB_FIELD (cb_ref (cb_call_params))->values = cb_list (cb_zero);
 
   /* TALLY */
   {
-    /* 01 TALLY GLOBAL PICTURE 9(5) USAGE BINARY VALUE ZERO. */
+    /* 01 TALLY GLOBAL PICTURE 9(9) USAGE COMP-5 VALUE ZERO. */
     cb_tree x = cb_build_field (cb_build_reference ("TALLY"));
-    CB_FIELD (x)->pic = CB_PICTURE (cb_build_picture ("9(5)"));
-    CB_FIELD (x)->usage = CB_USAGE_BINARY;
+    CB_FIELD (x)->pic = CB_PICTURE (cb_build_picture ("9(9)"));
+    CB_FIELD (x)->usage = CB_USAGE_COMP_5;
     CB_FIELD (x)->values = cb_list (cb_zero);
     cb_validate_field (CB_FIELD (x));
 
@@ -514,13 +519,22 @@ cb_tree
 cb_build_length (cb_tree x)
 {
   cb_tree temp;
+  struct cb_field *f;
 
   if (x == cb_error_node)
     return cb_error_node;
   if (CB_REFERENCE_P (x) && cb_ref (x) == cb_error_node)
     return cb_error_node;
 
+  f = CB_FIELD (cb_ref (x));
+  if (cb_field_variable_size (f) == NULL)
+    {
+	char buff[64];
+	sprintf(buff, "%d", cb_field_size(x));
+      return cb_build_numeric_literal (0, buff, 0);
+    }
   temp = cb_build_index (cb_build_filler ());
+  CB_FIELD (cb_ref (temp))-> usage = CB_USAGE_LENGTH;
   cb_emit (cb_build_assign (temp, cb_build_length_1 (x)));
   return temp;
 }
@@ -534,7 +548,7 @@ cb_build_address (cb_tree x)
   return cb_build_cast_address (x);
 }
 
-
+
 /* validate program */
 
 void
@@ -585,6 +599,7 @@ cb_validate_program_data (struct cb_program *prog)
 	  {
 	    cb_tree x = cb_build_implicit_field (assign, FILENAME_MAX);
 	    struct cb_field *p = current_program->working_storage;
+	    CB_FIELD(x)->count++;
 	    if (p)
 	      {
 		while (p->sister)
@@ -624,7 +639,7 @@ cb_validate_program_body (struct cb_program *prog)
   prog->exec_list = cb_list_reverse (prog->exec_list);
 }
 
-
+
 /*
  * Expressions
  */
@@ -980,7 +995,7 @@ cb_expr_finish (void)
   return expr_stack[3].value;
 }
 
-
+
 /*
  * Numerical operation
  */
@@ -1193,7 +1208,7 @@ cb_emit_arithmetic (cb_tree vars, char op, cb_tree val)
   cb_emit (build_decimal_assign (vars, op, val));
 }
 
-
+
 /*
  * Condition
  */
@@ -1331,7 +1346,7 @@ cb_build_cond (cb_tree x)
     }
 }
 
-
+
 /*
  * ADD/SUBTRACT CORRESPONDING
  */
@@ -1395,7 +1410,7 @@ cb_emit_corresponding (cb_tree (*func)(), cb_tree x1, cb_tree x2, cb_tree opt)
   emit_corresponding (func, x1, x2, opt);
 }
 
-
+
 /*
  * ACCEPT statement
  */
@@ -1468,6 +1483,18 @@ cb_emit_accept_environment (cb_tree var)
   cb_emit (cb_build_funcall_1 ("cob_accept_environment", var));
 }
 
+void
+cb_emit_accept_arg_number (cb_tree var)
+{
+  cb_emit (cb_build_funcall_1 ("cob_accept_arg_number", var));
+}
+
+void
+cb_emit_accept_arg_value (cb_tree var)
+{
+  cb_emit (cb_build_funcall_1 ("cob_accept_arg_value", var));
+}
+
 void
 cb_emit_accept_mnemonic (cb_tree var, cb_tree mnemonic)
 {
@@ -1509,7 +1536,7 @@ cb_emit_accept_name (cb_tree var, cb_tree name)
   cb_error_x (name, _("'%s' undefined in SPECIAL-NAMES"), CB_NAME (name));
 }
 
-
+
 /*
  * CALL statement
  */
@@ -1523,7 +1550,7 @@ cb_emit_call (cb_tree prog, cb_tree using, cb_tree returning,
     cb_emit (cb_build_move (cb_return_code, returning));
 }
 
-
+
 /*
  * CANCEL statement
  */
@@ -1534,7 +1561,7 @@ cb_emit_cancel (cb_tree prog)
   cb_emit (cb_build_funcall_1 ("cob_cancel", prog));
 }
 
-
+
 /*
  * CLOSE statement
  */
@@ -1547,7 +1574,7 @@ cb_emit_close (cb_tree file, cb_tree opt)
   cb_emit (cb_build_funcall_2 ("cob_close", file, opt));
 }
 
-
+
 /*
  * DELETE statement
  */
@@ -1560,7 +1587,7 @@ cb_emit_delete (cb_tree file)
   cb_emit (cb_build_funcall_1 ("cob_delete", file));
 }
 
-
+
 /*
  * DISPLAY statement
  */
@@ -1600,6 +1627,26 @@ cb_emit_display (cb_tree values, cb_tree upon, cb_tree no_adv, cb_tree pos)
 	}
       cb_emit (cb_build_funcall_1 ("cob_display_environment", CB_VALUE (values)));
     }
+  else if (upon == cb_int2)
+    {
+      /* DISPLAY x UPON ENVIRONMENT-VALUE */
+      if (cb_list_length (values) != 1)
+	{
+	  cb_error (_("wrong number of data items"));
+	  return;
+	}
+      cb_emit (cb_build_funcall_1 ("cob_display_env_value", CB_VALUE (values)));
+    }
+  else if (upon == cb_int3)
+    {
+      /* DISPLAY x UPON ARGUMENT-NUMBER */
+      if (cb_list_length (values) != 1)
+	{
+	  cb_error (_("wrong number of data items"));
+	  return;
+	}
+      cb_emit (cb_build_funcall_1 ("cob_display_arg_number", CB_VALUE (values)));
+    }
   else
     {
       /* DISPLAY x ... [UPON device-name] */
@@ -1661,7 +1708,7 @@ cb_build_display_upon_direct (cb_tree x)
   return cb_error_node;
 }
 
-
+
 /*
  * DIVIDE statement
  */
@@ -1685,7 +1732,7 @@ cb_emit_divide (cb_tree dividend, cb_tree divisor,
 						   cb_int0)));
 }
 
-
+
 /*
  * EVALUATE statement
  */
@@ -1784,7 +1831,7 @@ cb_emit_evaluate (cb_tree subject_list, cb_tree case_list)
   cb_emit (build_evaluate (subject_list, case_list));
 }
 
-
+
 /*
  * GO TO statement
  */
@@ -1816,7 +1863,7 @@ cb_emit_exit (void)
   cb_emit (cb_build_goto (0, 0));
 }
 
-
+
 /*
  * IF statement
  */
@@ -1827,7 +1874,7 @@ cb_emit_if (cb_tree cond, cb_tree stmt1, cb_tree stmt2)
   cb_emit (cb_build_if (cond, stmt1, stmt2));
 }
 
-
+
 /*
  * INITIALIZE statement
  */
@@ -1842,7 +1889,7 @@ cb_emit_initialize (cb_tree vars, cb_tree value, cb_tree replacing, cb_tree def)
     cb_emit (cb_build_initialize (CB_VALUE (l), value, replacing, def));
 }
 
-
+
 /*
  * INSPECT statement
  */
@@ -1953,7 +2000,7 @@ cb_build_inspect_region (cb_tree l, cb_tree pos, cb_tree x)
     return cb_list_add (l, cb_build_funcall_1 ("cob_inspect_after", x));
 }
 
-
+
 /*
  * MOVE statement
  */
@@ -2507,6 +2554,15 @@ cb_build_move (cb_tree src, cb_tree dst)
   if (src == cb_error_node || dst == cb_error_node)
     return cb_error_node;
 
+  if (CB_TREE_CLASS (src) == CB_CLASS_POINTER
+      && CB_REFERENCE_P(src)
+      && cb_field(src)->level > 50 )
+	cb_error_x (src, _("invalid MOVE statement"));
+  if (CB_TREE_CLASS (dst) == CB_CLASS_POINTER
+      && CB_REFERENCE_P(dst)
+      && cb_field(dst)->level > 50 )
+	cb_error_x (dst, _("invalid MOVE statement"));
+
   validate_move (src, dst, 0);
 
   if (CB_REFERENCE_P (src))
@@ -2571,7 +2627,7 @@ cb_emit_move (cb_tree src, cb_tree dsts)
     cb_emit (cb_build_move (src, CB_VALUE (l)));
 }
 
-
+
 /*
  * OPEN statement
  */
@@ -2592,7 +2648,7 @@ cb_emit_open (cb_tree file, cb_tree mode, cb_tree sharing)
   cb_emit (cb_build_funcall_3 ("cob_open", file, mode, sharing));
 }
 
-
+
 /*
  * PERFORM statement
  */
@@ -2642,7 +2698,7 @@ cb_build_perform_exit (struct cb_label *label)
   return x;
 }
 
-
+
 /*
  * READ statement
  */
@@ -2670,7 +2726,7 @@ cb_emit_read (cb_tree ref, cb_tree next, cb_tree into, cb_tree key)
   current_statement->file = file;
 }
 
-
+
 /*
  * REWRITE statement
  */
@@ -2685,7 +2741,7 @@ cb_emit_rewrite (cb_tree record, cb_tree from)
   cb_emit (cb_build_funcall_2 ("cob_rewrite", file, record));
 }
 
-
+
 /*
  * RETURN statement
  */
@@ -2701,7 +2757,7 @@ cb_emit_return (cb_tree ref, cb_tree into)
   current_statement->file = file;
 }
 
-
+
 /*
  * SEARCH statement
  */
@@ -2785,7 +2841,7 @@ cb_emit_search_all (cb_tree table, cb_tree at_end, cb_tree when, cb_tree stmts)
 					 stmts, NULL)));
 }
 
-
+
 /*
  * SET statement
  */
@@ -2876,7 +2932,7 @@ cb_emit_set_true (cb_tree l)
     }
 }
 
-
+
 /*
  * SORT statement
  */
@@ -2970,7 +3026,7 @@ cb_emit_sort_finish (cb_tree file)
     cb_emit (cb_build_funcall_1 ("cob_sort_finish", cb_ref (file)));
 }
 
-
+
 /*
  * START statement
  */
@@ -2983,7 +3039,7 @@ cb_emit_start (cb_tree file, cb_tree op, cb_tree key)
 			       key ? key : CB_FILE (cb_ref (file))->key));
 }
 
-
+
 /*
  * STOP statement
  */
@@ -2994,7 +3050,7 @@ cb_emit_stop_run (cb_tree x)
   cb_emit (cb_build_funcall_1 ("cob_stop_run", cb_build_cast_integer (x)));
 }
 
-
+
 /*
  * STRING statement
  */
@@ -3027,7 +3083,7 @@ cb_emit_string (cb_tree items, cb_tree into, cb_tree pointer)
   cb_emit (cb_build_funcall_0 ("cob_string_finish"));
 }
 
-
+
 /*
  * UNSTRING statement
  */
@@ -3062,7 +3118,7 @@ cb_build_unstring_into (cb_tree name, cb_tree delimiter, cb_tree count)
   return cb_build_funcall_3 ("cob_unstring_into", name, delimiter, count);
 }
 
-
+
 /*
  * WRITE statement
  */
@@ -3072,8 +3128,8 @@ cb_emit_write (cb_tree record, cb_tree from, cb_tree opt)
 {
   struct cb_field *f = CB_FIELD (cb_ref (record));
   cb_tree file = CB_TREE (f->file);
-  current_statement->file = file;
 
+  current_statement->file = file;
   if (from)
     cb_emit (cb_build_move (from, record));
   cb_emit (cb_build_funcall_3 ("cob_write", file, record, opt));
