@@ -37,11 +37,17 @@
 int cob_argc = 0;
 char **cob_argv = NULL;
 
+int cob_initialized = 0;
+int cob_exception_code = 0;
+
+cob_module *cob_current_module = NULL;
+
 const char *cob_source_file = NULL;
 unsigned int cob_source_line = 0;
 const char *cob_source_statement = NULL;
 
 int cob_linage_counter = 0;
+int cob_call_params = 0;
 
 static cob_field_attr all_attr = {COB_TYPE_ALPHANUMERIC_ALL, 0, 0, 0, NULL};
 
@@ -51,7 +57,7 @@ cob_field cob_high =  {1, "\xff", &all_attr};
 cob_field cob_low =   {1, "\0",   &all_attr};
 cob_field cob_quote = {1, "\"",   &all_attr};
 
-const long cob_exp10[10] = {
+const int cob_exp10[10] = {
   1,
   10,
   100,
@@ -121,6 +127,7 @@ const unsigned char cob_a2e[256] = {
   0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
 };
 
+/* Not needed at the moment - comment out
 const unsigned char cob_e2a[256] = {
   0x00, 0x01, 0x02, 0x03, 0x9c, 0x09, 0x86, 0x7f,
   0x97, 0x8d, 0x8e, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
@@ -155,10 +162,7 @@ const unsigned char cob_e2a[256] = {
   0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
   0x38, 0x39, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
 };
-
-cob_module *cob_current_module = NULL;
-
-int cob_exception_code;
+end of comment out */
 
 struct cob_exception cob_exception_table[] = {
   {0, 0, 0},		/* COB_EC_ZERO */
@@ -168,9 +172,7 @@ struct cob_exception cob_exception_table[] = {
   {0, 0, 0}		/* COB_EC_MAX */
 };
 
-int cob_initialized = 0;
 
-
 /*
  * General functions
  */
@@ -223,7 +225,7 @@ cob_stop_run (int status)
   exit (status);
 }
 
-
+
 /*
  * Sign
  */
@@ -334,7 +336,7 @@ cob_real_put_sign (cob_field *f, int sign)
     }
 }
 
-
+
 char *
 cob_field_to_string (cob_field *f, char *s)
 {
@@ -347,7 +349,7 @@ cob_field_to_string (cob_field *f, char *s)
   return s;
 }
 
-
+
 /*
  * Switch
  */
@@ -366,7 +368,7 @@ cob_set_switch (int n, int flag)
   cob_switch[n] = flag;
 }
 
-
+
 /*
  * Comparison
  */
@@ -535,7 +537,7 @@ cob_cmp_int (cob_field *f1, int n)
   return cob_numeric_cmp (f1, &temp);
 }
 
-
+
 /*
  * Class check
  */
@@ -622,7 +624,7 @@ cob_is_lower (cob_field *f)
   return 1;
 }
 
-
+
 /*
  * Table sort
  */
@@ -670,7 +672,7 @@ cob_table_sort (cob_field *f, int n)
   qsort (f->data, n, f->size, sort_compare);
 }
 
-
+
 /*
  * Run-time error checking
  */
@@ -761,3 +763,29 @@ cob_check_ref_mod (int offset, int length, int size, const char *name)
       exit (1);
     }
 }
+
+char *
+cob_external_addr (char *exname, int exlength)
+{
+	static cob_external *basext = NULL;
+
+	cob_external	*eptr;
+
+	for ( eptr = basext; eptr; eptr = eptr->next ) {
+		if ( !strcmp(exname, eptr->ename) ) {
+			if ( exlength > eptr->esize ) {
+				cob_runtime_error (_("EXTERNAL item '%s' has size > %d"), exname, exlength);
+				exit (1);
+			}
+			return eptr->ext_alloc;
+		}
+	}
+	eptr = (cob_external *)malloc(sizeof(cob_external));
+	eptr->next = basext;
+	eptr->esize = exlength;
+	eptr->ename = malloc(strlen(exname) + 1);
+	strcpy(eptr->ename, exname);
+	eptr->ext_alloc = malloc(exlength);
+	basext = eptr;
+	return eptr->ext_alloc;
+}
