@@ -51,7 +51,7 @@ store_common_region (cob_field *f, unsigned char *data, size_t size, int scale)
     memset (f->data, '0', f->size);
 }
 
-
+
 /*
  * Display
  */
@@ -194,7 +194,7 @@ cob_move_alphanum_to_alphanum (cob_field *f1, cob_field *f2)
     }
 }
 
-
+
 /*
  * Packed decimal
  */
@@ -254,7 +254,7 @@ cob_move_packed_to_display (cob_field *f1, cob_field *f2)
   cob_put_sign (f2, sign);
 }
 
-
+
 /*
  * Binary integer
  */
@@ -316,7 +316,7 @@ cob_move_binary_to_display (cob_field *f1, cob_field *f2)
   cob_put_sign (f2, sign);
 }
 
-
+
 /*
  * Edited
  */
@@ -572,7 +572,7 @@ cob_move_alphanum_to_edited (cob_field *f1, cob_field *f2)
   cob_put_sign (f1, sign);
 }
 
-
+
 /*
  * MOVE dispatcher
  */
@@ -701,7 +701,7 @@ cob_move (cob_field *src, cob_field *dst)
     }
 }
 
-
+
 /*
  * Convenience functions
  */
@@ -750,6 +750,7 @@ long long
 cob_binary_get_int64 (cob_field *f)
 {
   long long n = 0;
+  int fsiz = 8 - f->size;
 #ifndef WORDS_BIGENDIAN
   if (COB_FIELD_BINARY_SWAP (f))
     {
@@ -757,11 +758,11 @@ cob_binary_get_int64 (cob_field *f)
 	{
 	  memcpy (&n, f->data, f->size);
 	  n = COB_BSWAP_64 (n);
-	  n >>= 8 * (8 - f->size); /* shift with sign */
+	  n >>= 8 * fsiz; /* shift with sign */
 	}
       else
 	{
-	  memcpy (((unsigned char *) &n) + 8 - f->size, f->data, f->size);
+	  memcpy (((unsigned char *) &n) + fsiz, f->data, f->size);
 	  n = COB_BSWAP_64 (n);
 	}
     }
@@ -769,8 +770,8 @@ cob_binary_get_int64 (cob_field *f)
     {
       if (COB_FIELD_HAVE_SIGN (f))
 	{
-	  memcpy (((unsigned char *) &n) + 8 - f->size, f->data, f->size);
-	  n >>= 8 * (8 - f->size); /* shift with sign */
+	  memcpy (((unsigned char *) &n) + fsiz, f->data, f->size);
+	  n >>= 8 * fsiz; /* shift with sign */
 	}
       else
 	{
