@@ -41,7 +41,7 @@ static cob_decimal cob_d2;
 static cob_decimal cob_d3;
 static cob_decimal cob_d4;
 
-
+
 /*
  * Decimal number
  */
@@ -53,12 +53,6 @@ cob_decimal_init (cob_decimal *d)
   d->scale = 0;
 }
 
-void
-cob_decimal_clear (cob_decimal *d)
-{
-  mpz_clear (d->value);
-}
-
 void
 cob_decimal_print (cob_decimal *d)
 {
@@ -114,7 +108,7 @@ align_decimal (cob_decimal *d1, cob_decimal *d2)
     shift_decimal (d2, d1->scale - d2->scale);
 }
 
-
+
 /*
  * Decimal set/get
  */
@@ -269,11 +263,11 @@ cob_decimal_get_binary (cob_decimal *d, cob_field *f, int opt)
 {
   int overflow = 0;
   int digits = f->attr->digits;
-  if (f->size <= sizeof (long))
+  if (f->size <= 4)
     {
       if (COB_FIELD_HAVE_SIGN (f))
 	{
-	  long val;
+	  int val;
 	  if (!mpz_fits_sint_p (d->value))
 	    {
 	      if ( cob_current_module->flag_binary_truncate ) {
@@ -309,7 +303,7 @@ cob_decimal_get_binary (cob_decimal *d, cob_field *f, int opt)
 	}
       else
 	{
-	  unsigned long val;
+	  unsigned int val;
 	  if (mpz_sgn (d->value) < 0)
 	    mpz_abs (d->value, d->value);
 	  if (!mpz_fits_uint_p (d->value))
@@ -344,7 +338,7 @@ cob_decimal_get_binary (cob_decimal *d, cob_field *f, int opt)
     }
   else
     {
-      unsigned long lo;
+      unsigned int lo;
       mpz_fdiv_r_2exp (cob_d2.value, d->value, 32);
       mpz_fdiv_q_2exp (d->value, d->value, 32);
       lo = mpz_get_ui (cob_d2.value);
@@ -522,7 +516,7 @@ cob_decimal_get_field (cob_decimal *d, cob_field *f, int opt)
     }
 }
 
-
+
 /*
  * Decimal arithmetic
  */
@@ -594,7 +588,7 @@ cob_decimal_cmp (cob_decimal *d1, cob_decimal *d2)
   return mpz_cmp (d1->value, d2->value);
 }
 
-
+
 /*
  * Optimized arithmetic for DISPLAY
  */
@@ -772,7 +766,7 @@ cob_display_add_int (cob_field *f, int n)
   return cob_exception_code;
 }
 
-
+
 /*
  * Convenience functions
  */
@@ -863,7 +857,7 @@ cob_numeric_cmp (cob_field *f1, cob_field *f2)
   return cob_decimal_cmp (&cob_d1, &cob_d2);
 }
 
-
+
 void
 cob_init_numeric (void)
 {
